"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const ethereumjs_account_1 = require("ethereumjs-account");
const ethereumjs_util_1 = require("ethereumjs-util");
const fs = require("fs-extra");
class Account extends command_1.Command {
    async run() {
        const Trie = require('merkle-patricia-tree/secure');
        const level = require('level');
        const { args, flags } = this.parse(Account);
        let workDir = flags.dbdir;
        if (!flags['skip-copy']) {
            workDir = flags['work-dir'] || '.ethstatedb';
            await fs.emptyDir(workDir);
            await fs.copy(flags.dbdir, workDir);
        }
        const db = level(workDir);
        const trie = new Trie(db, flags.root);
        const found = await new Promise((resolve, reject) => {
            trie.checkRoot(flags.root, (err, value) => {
                if (err)
                    return reject(err);
                resolve(value);
            });
        });
        if (!found)
            return this.warn(`stateRoot (${flags.root}) is not found in db`);
        const account = await new Promise((resolve, reject) => {
            trie.get(args.address, (err, raw) => {
                if (err)
                    return reject(err);
                return resolve(new ethereumjs_account_1.default(raw));
            });
        });
        this.log('');
        this.print(args.address, account);
    }
    print(address, account) {
        cli_ux_1.default.table([
            { name: 'Address', value: address },
            {
                name: 'Balance',
                value: `${new ethereumjs_util_1.BN(account.balance).toString()}`,
            },
            { name: 'Nonce', value: `${new ethereumjs_util_1.BN(account.nonce).toString()}` },
            {
                name: 'StateRoot',
                value: `${account.stateRoot.toString('hex') === ethereumjs_util_1.KECCAK256_RLP_S
                    ? ''
                    : '0x' + account.stateRoot.toString('hex')}`,
            },
            {
                name: 'CodeHash',
                value: `${account.codeHash.toString('hex') === ethereumjs_util_1.KECCAK256_NULL_S
                    ? ''
                    : '0x' + account.codeHash.toString('hex')}`,
            },
        ], { name: {}, value: {} }, { printLine: this.log, 'no-header': true });
    }
}
Account.description = 'load information of an account';
Account.examples = [
    '$ ethstatedb account --dbdir /home/data/ethereum-node1/geth/chaindata --root 0x595e3a0db7f38600770a267b12b67a8da1c202fef4b6c21e60fdb92477d89753 0x7C476EA4e20D6d49820Ee810C0C474ED11b3f527',
];
Account.flags = {
    help: command_1.flags.help({ char: 'h' }),
    dbdir: command_1.flags.string({
        char: 'd',
        required: true,
        description: 'path of Ethereum state database, e.g. ethereum/geth/chaindata',
    }),
    root: command_1.flags.string({
        char: 'r',
        required: true,
        description: 'value of stateRoot',
    }),
    'skip-copy': command_1.flags.boolean({
        required: false,
        description: 'skip copying database to .ethstatedb. By default, database is copied to avoid messing the original database.',
    }),
    'work-dir': command_1.flags.string({
        required: false,
        description: 'directory to pull down database',
    }),
};
Account.args = [
    {
        name: 'address',
        required: true,
        description: 'public key of an account, e.g. 0x7C476EA4e20D6d49820Ee810C0C474ED11b3f527',
    },
];
exports.default = Account;
